<?php

namespace Drupal\mydropwizard\Commands;

use Drupal\Core\Extension\ModuleExtensionList;
use Drupal\Core\Extension\ThemeExtensionList;
use Drupal\mydropwizard\MydropwizardVitalsSender;
use Drush\Commands\DrushCommands;
use Consolidation\OutputFormatters\StructuredData\RowsOfFields;
use Symfony\Component\Console\Input\InputDefinition;

class MydropwizardDrushCommands extends DrushCommands {

  /**
   * @var \Drupal\Core\Extension\ModuleExtensionList
   */
  protected $moduleList;

  /**
   * @var \Drupal\Core\Extension\ThemeExtensionList
   */
  protected $themeList;

  /**
   * @var \Drupal\mydropwizard\MydropwizardVitalsSender
   */
  protected $vitalsSender;

  /**
   * MydropwizardDrushCommands constructor.
   *
   * @param \Drupal\mydropwizard\MydropwizardVitalsSender $vitals_sender
   *   The vitals sender service.
   */
  public function __construct(ModuleExtensionList $module_list, ThemeExtensionList $theme_list, MydropwizardVitalsSender $vitals_sender) {
    $this->moduleList = $module_list;
    $this->themeList = $theme_list;
    $this->vitalsSender = $vitals_sender;
  }

  /**
   * Outputs the list of enabled projects on this site.
   *
   * @command mydropwizard:list-projects
   * @aliases mydropwizard-list-projects
   * @option branch-only
   *   Only output the major branch version - not the full version string
   * @usage drush mydropwizard:list-projects
   *   Show a list of enabled projects and their versions in a table.
   * @usage drush mydropwizard:list-projects --format=csv
   *   Show the same list but as CSV.
   * @table-style default
   * @field-labels
   *   project: Project
   *   version: Version
   * @default-fields project,version
   * @return \Consolidation\OutputFormatters\StructuredData\RowsOfFields
   */
  public function listProjects($options = ['branch-only' => FALSE]) {
    $module_list = $this->moduleList->reset()->getList();
    $theme_list = $this->themeList->reset()->getList();

    $project_data = $this->vitalsSender->gatherProjectData($module_list, $theme_list);

    $data = [];
    foreach ($project_data as $project) {
      $name = $project['name'];
      $version = $project['value'];

      // Switch to only major branch version.
      if ($options['branch-only']) {
        // Extract the major version part.
        if (preg_match('/^(\d+)/', $version, $matches)) {
          $version = $matches[1];
        }
        else {
          $version = '0';
        }
      }

      $data[] = array(
        'project' => $name,
        'version' => $version,
      );
    }

    return new RowsOfFields($data);
  }

  // The following functions are necessary for Drush 8 compatibility in some
  // cases, where it's detecting this as a Drupal Console command. Drush 8 will
  // also pick up the mydropwizard.drush.inc which provides the real command.

  public function getName() {
    return 'mydropwizard:invalid';
  }

  public function getAliases() {
    return [];
  }

  public function getDescription() {
    return 'Invalid command that appears in some cases with Drush 8 - please ignore!';
  }

  public function getUsages() {
    return [];
  }

  public function getDefinition() {
    return new InputDefinition();
  }

}
