<?php


namespace Drupal\mydropwizard;

use Drupal\Core\Config\ConfigFactoryInterface;
use GuzzleHttp\Client as HttpClient;
use Psr\Log\LoggerInterface;

/**
 * Service for accessing the myDropWizard monitoring API.
 */
class MydropwizardApi {

  const ENDPOINT_BASE = 'https://portal.mydropwizard.com/api/v1';

  const SETTINGS = 'mydropwizard.settings';

  /**
   * The authentication token.
   *
   * Can be one of three values:
   * - NULL if we haven't tried to get it yet
   * - FALSE if we tried and failed
   * - The token string
   *
   * @var null|bool|string
   */
  protected $token = NULL;

  /**
   * @var \GuzzleHttp\Client
   */
  protected $httpClient;

  /**
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * MydropwizardApi constructor.
   *
   * @param \GuzzleHttp\Client $http_client
   *   The HTTP client.
   */
  public function __construct(HttpClient $http_client, ConfigFactoryInterface $config_factory, LoggerInterface $logger) {
    $this->httpClient = $http_client;
    $this->configFactory = $config_factory;
    $this->logger = $logger;
  }

  /**
   * Checks that we have enough data to make an API call.
   */
  public function isReady() {
    $token = $this->getToken();
    return !empty($token);
  }

  /**
   * Gets the configuration necessary for interacting with the API.
   */
  public function getConfiguration() {
    $config = $this->configFactory->get(static::SETTINGS);

    return [
      'username' => $config->get('username'),
      'password' => $config->get('api_key'),
      'mode' => $config->get('submission_mode'),
      'site_id' => $config->get('site_id'),
    ];
  }

  /**
   * Checks that we have the necessary configuration.
   */
  protected function checkConfiguration($config) {
    if (empty($config['username']) || empty($config['password']) || empty($config['mode'])) {
      return FALSE;
    }
    if ($config['mode'] === 'push' && empty($config['site_id'])) {
      return FALSE;
    }
    return TRUE;
  }

  /**
   * Gets the absolute API URL for a given path.
   */
  protected function getUrl($path) {
    $config = $this->configFactory->get(static::SETTINGS);
    $base_url = $config->get('api_endpoint');
    if (empty($base_url)) {
      $base_url = static::ENDPOINT_BASE;
    }
    return rtrim($base_url, '/') . '/' . $path;
  }

  /**
   * Gets authorization token.
   */
  protected function getToken() {
    if ($this->token === NULL) {
      $config = $this->getConfiguration();
      if (!$this->checkConfiguration($config)) {
        $this->logger->error('Incomplete configuration - check your settings!');
        $this->token = FALSE;
      }
      else {
        $this->token = base64_encode("{$config['username']}:{$config['password']}");
      }
    }

    return $this->token;
  }

  /**
   * Makes a request to the API.
   *
   * @param string $method
   *   The HTTP method.
   * @param string $path
   *   The relative path to the API.
   * @param array $options
   *   The request options, same as Guzzle takes.
   *
   * @return \Psr\Http\Message\ResponseInterface
   *   The response, same as Guzzle returns.
   *
   * @throws \RuntimeException
   */
  public function request($method, $path, $options = array()) {
    $options['headers']['Content-Type'] = 'application/json';
    $options['headers']['Accept'] = 'application/json';

    if ($token = $this->getToken()) {
      $options['headers']['Authorization'] = 'Basic ' . $token;
    }

    return $this->httpClient->request($method, $this->getUrl($path), $options);
  }

}