<?php


namespace Drupal\mydropwizard;


use Drupal\Core\Extension\Extension;
use Drupal\Core\Extension\ModuleExtensionList;
use Drupal\Core\Extension\ThemeExtensionList;
use Drupal\system\SystemManager;
use Psr\Log\LoggerInterface;

/**
 * Sends vitals data for this site to myDropWizard.
 */
class MydropwizardVitalsSender {

  /**
   * @var \Drupal\mydropwizard\MydropwizardApi
   */
  protected $api;

  /**
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * @var \Drupal\Core\Extension\ModuleExtensionList
   */
  protected $moduleList;

  /**
   * @var \Drupal\Core\Extension\ThemeExtensionList
   */
  protected $themeList;

  /**
   * @var \Drupal\system\SystemManager
   */
  protected $systemManager;

  /**
   * @var array
   */
  protected $requirement_severity_map;

  /**
   * MydropwizardVitalsSender constructor.
   *
   * @param \Drupal\mydropwizard\MydropwizardApi $api
   *   myDropWizard API services.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger for the 'mydropwizard' channel.
   * @param \Drupal\Core\Extension\ModuleExtensionList $module_list
   *   The module list.
   * @param \Drupal\Core\Extension\ThemeExtensionList $theme_list
   *   The theme list.
   * @param \Drupal\system\SystemManager $system_manager
   *   The system manager.
   */
  public function __construct(MydropwizardApi $api, LoggerInterface $logger, ModuleExtensionList $module_list, ThemeExtensionList $theme_list, SystemManager $system_manager) {
    $this->api = $api;
    $this->logger = $logger;
    $this->moduleList = $module_list;
    $this->themeList = $theme_list;
    $this->systemManager = $system_manager;
  }

  /**
   * Sends vitals data to myDropWizard.
   *
   * @param string $payload_id
   *   The payload id when using the 'pull' submission mode; NULL when using
   *   the 'push' submission mode.
   * @param bool $rebuild
   *   If TRUE, rebuild the extension lists before gathering vitals.
   */
  public function sendVitalsData($payload_id = NULL, $rebuild = TRUE) {
    if (!$this->api->isReady()) {
      return;
    }

    $data = $this->gatherAllVitalsData($rebuild);

    if ($payload_id) {
      $response = $this->api->request('PUT', 'payload/' . $payload_id, array(
        'body' => json_encode($data, TRUE),
      ));
    }
    else {
      $api_config = $this->api->getConfiguration();
      $data['site_id'] = $api_config['site_id'];
      $response = $this->api->request('POST', 'payload', array(
        'body' => json_encode($data, TRUE),
      ));
    }


    if ($response->getStatusCode() != 200 && $response->getStatusCode() != 201) {
      $this->logger->error('Error sending vitals: {code} {error}', ['code' => $response->getStatusCode(), 'error' => $response->getBody()]);
    }
    else {
      $this->logger->info('Vitals sent');
    }
  }

  protected function gatherAllVitalsData($rebuild = TRUE) {
    $data = [];

    if ($rebuild) {
      $this->moduleList->reset();
      $this->themeList->reset();
    }
    $module_list = $this->moduleList->getList();
    $theme_list = $this->themeList->getList();

    $data['vitals'] = [
      'project' => $this->gatherProjectData($module_list, $theme_list),
      'module' => $this->gatherModuleData($module_list),
      'theme' => $this->gatherThemeData($theme_list),
      'library' => $this->gatherLibraryData(),
      'requirement' => $this->gatherRequirementData(),
    ];

    return $data;
  }

  /**
   * Gathers project data.
   *
   * @param \Drupal\Core\Extension\Extension[] $module_list
   *   The module list.
   * @param \Drupal\Core\Extension\Extension[] $theme_list
   *   The theme list.
   *
   * @return array
   */
  public function gatherProjectData(array $module_list, array $theme_list) {
    $projects = [];

    foreach ($module_list as $module) {
      if ($module->status) {
        $this->processProject($projects, $module);
      }
    }
    foreach ($theme_list as $theme) {
      if ($theme->status) {
        $this->processProject($projects, $theme);
      }
    }

    $projects['drupal'] = [
      'name' => 'drupal',
      'value' => \Drupal::VERSION,
    ];

    return array_values($projects);
  }

  /**
   * Processes an individual module or theme into project data.
   *
   * @param array &$projects
   *   The projects array to add projects to.
   * @param \Drupal\Core\Extension\Extension $extension
   *   The module or theme.
   *
   * @return void
   */
  protected function processProject(array &$projects, Extension $extension) {
    if (empty($extension->info)) {
      return;
    }

    $project_name = $this->getProjectName($extension);
    $version = !empty($extension->info['version']) ? $extension->info['version'] : '1.x-dev';

    // Remove core compatibility from version numbers.
    foreach (['8.x-', '9.x-'] as $core_compatibility) {
      if (strpos($version, $core_compatibility) === 0) {
        $version = substr($version, strlen($core_compatibility));
      }
    }

    $projects[$project_name] = [
      'name' => $project_name,
      'value' => (string) $version,
    ];
  }

  /**
   * Gets the project name for an extension.
   *
   * @param \Drupal\Core\Extension\Extension $extension
   *   The extension.
   *
   * @return string
   *   The project name.
   */
  protected function getProjectName(Extension $extension) {
    if (!empty($extension->info['project'])) {
      return $extension->info['project'];
    }
    if (strpos($extension->getPath(), 'core/') === 0) {
      return 'drupal';
    }
    // If we have no other ideas, use the module name.
    return $extension->getName();
  }

  /**
   * Gathers module data.
   *
   * @param \Drupal\Core\Extension\Extension[] $module_list
   *   The module list.
   *
   * @return array
   */
  protected function gatherModuleData(array $module_list) {
    $modules = array();

    foreach ($module_list as $module) {
      if (empty($module->info['hidden'])) {
        $modules[$module->getName()] = [
          'name' => $module->getName(),
          'value' => !empty($module->status) ? 'enabled' : 'disabled',
        ];
      }
    }

    return array_values($modules);
  }

  /**
   * Gathers theme data.
   *
   * @param \Drupal\Core\Extension\Extension[] $theme_list
   *   The theme list.
   *
   * @return array
   */
  protected function gatherThemeData(array $theme_list) {
    $themes = array();

    foreach ($theme_list as $theme) {
      if (empty($theme->info['hidden'])) {
        $themes[$theme->getName()] = [
          'name' => $theme->getName(),
          'value' => !empty($theme->status) ? 'enabled' : 'disabled',
        ];
      }
    }

    return array_values($themes);
  }

  /**
   * Gathers library data.
   *
   * @return array
   */
  protected function gatherLibraryData() {
    return [];
  }

  /**
   * Gathers requirement data.
   *
   * @return array
   */
  protected function gatherRequirementData() {
    // Load for the REQUIREMENT_* constants.
    require_once DRUPAL_ROOT . '/core/includes/install.inc';

    if ($this->requirement_severity_map === NULL) {
      $this->requirement_severity_map = array(
        REQUIREMENT_INFO => 'info',
        REQUIREMENT_OK => 'ok',
        REQUIREMENT_WARNING => 'warning',
        REQUIREMENT_ERROR => 'error',
      );
    }

    $requirements = [];
    foreach ($this->systemManager->listRequirements() as $name => $raw_requirement) {
      $requirement = array(
        'name' => (string) $name,
        'value' => (string) $raw_requirement['value'],
      );

      if (!empty($raw_requirement['severity'])) {
        $requirement['status'] = $this->requirement_severity_map[$raw_requirement['severity']];
      }
      else {
        $requirement['status'] = 'info';
      }

      // Cleanup the PHP version (Drupal puts some HTML in there).
      if ($name === 'php') {
        $requirement['value'] = phpversion();
      }

      $requirements[] = $requirement;
    }

    return $requirements;
  }

}
